! ===========================================================================
MODULE spt_story_likelihood
! ===========================================================================

  use highell_options
  use highell_subroutines
  use foregrounds_loading

  implicit none
  REAL(8) ::  btt_dat(0:bmax0_k-1),btt_var(0:bmax0_k-1)
  REAL(8) ::  inverse(1:bmax0_k,1:bmax0_k),bval(0:bmax0_k-1)
  REAL(8), dimension (:), allocatable :: cl_c,cl_src
  REAL(8), dimension(:,:), allocatable :: win_func
  
  PRIVATE
  public :: spt_story_likelihood_init
  public :: spt_story_likelihood_compute
  contains
  
  ! ===========================================================================
  SUBROUTINE spt_story_likelihood_init
  ! ===========================================================================
    
    IMPLICIT NONE
    
    INTEGER  :: i,j,lun,il
    REAL(8)  :: dummy,ii
    CHARACTER(LEN=240) :: ttfilename, bblfilename, invcovfilename
    LOGICAL  :: good
    
    allocate(cl_c(2:tt_lmax_k),cl_src(2:tt_lmax_k))

    !-----------------------------------------------
    ! set file names
    !-----------------------------------------------
    
    ttfilename     = trim(data_dir)//trim(SPT_dir)//'spt_s12/spt_s12_150x150.dat'
    invcovfilename = trim(data_dir)//trim(SPT_dir)//'spt_s12/inverse_short.txt'
    bblfilename    = trim(data_dir)//trim(SPT_dir)//'spt_s12/BblMean_150x150.dat'

    !-----------------------------------------------------------
    ! load spectrum, covariance and band power window functions 
    !-----------------------------------------------------------
    
    inquire(file=ttfilename,exist = good)
    if(.not.good)then
       write(*,*) 'cant find', trim(ttfilename), trim(data_dir)//trim(SPT_dir)
       stop
    endif
    call get_free_lun( lun )
    open(unit=lun,file=ttfilename,form='formatted',status='unknown',action='read')    
    do i=0,bmax0_k-1
       read(lun,*) bval(i),btt_dat(i), btt_var(i)
    enddo
    close(lun)

    call get_free_lun( lun )
    open(unit=lun,file=invcovfilename,form='formatted',status='unknown',action='read')
    do i=1,bmax0_k
       read(lun,*) inverse(i,1:bmax0_k)
    enddo
    close(lun)

    inquire (file=bblfilename,exist = good)
    if(.not.good)then
       write(*,*) 'cant find', trim(bblfilename), trim(data_dir)//trim(SPT_dir)
       stop
    endif
    call get_free_lun( lun )
    open(unit=lun,file=bblfilename,form='formatted',status='unknown',action='read')
    allocate(win_func(0:bmax0_k-1,1:tt_lmax_k))
    do il = 2, tt_lmax_k
        read(lun,*) ii, (win_func(i,il), i=0,bmax0_k-1) 
        enddo
    close(lun)

  END SUBROUTINE spt_story_likelihood_init
  
  ! ===============================================================================
  SUBROUTINE spt_story_likelihood_compute(cltt,amp_tsz,amp_ksz,xi,amp_d,amp_s2,amp_c,n,beta_c,beta_d,alpha_s,cal_2,like_sptk)
  ! ===============================================================================
    
    IMPLICIT NONE
    REAL(8), intent(in) :: cltt(2:*), amp_tsz,amp_ksz,xi,amp_d,amp_s2
    REAL(8), intent(in) :: amp_c,n,beta_c,beta_d,alpha_s,cal_2
    REAL(8), intent(out) :: like_sptk
    INTEGER :: lun,il,i   
    REAL(8) :: cltt_temp(2:tt_lmax_k)
    REAL(8) :: btt_th(0:bmax0_k-1)
    REAL(8) :: diffs(bmax0_k,1),tmp(bmax0_k,1),diffs2(1,bmax0_k),chi2(1,1)
    REAL(8) :: feff,f2,f0,fcal_j
    REAL(8) :: sz_corr, planckfunctionratio_corr,flux2tempratio_corr
    REAL(8) :: f2_sz,f2_synch,f2_dust
    REAL(8) :: planckratiod2,fluxtempd2,fluxtemps2

    feff      =150.d0

!Define effective frequencies for SPT
    f2_sz     =152.9d0
    f2_synch  =150.2d0
    f2_dust   =153.8d0

    call sz_func(f2_sz,sz_corr)
    f2 = sz_corr
    call sz_func(feff,sz_corr)
    f0 = sz_corr
    call planckfunctionratio(f2_dust,feff,planckfunctionratio_corr)
    planckratiod2 = planckfunctionratio_corr
    call flux2tempratio(f2_dust,feff,flux2tempratio_corr)
    fluxtempd2 = flux2tempratio_corr
    call flux2tempratio(f2_synch,feff,flux2tempratio_corr)
    fluxtemps2 = flux2tempratio_corr

    !----------------------------------------------------------------
    ! Define CIB term
    !----------------------------------------------------------------
    do il=2,tt_lmax_k
       cl_c(il)=(il*(il+1)/3000.d0**2.d0)*(il/3000.d0)**(-n)
    enddo
    
    !----------------------------------------------------------------
    ! Calculate theory as C^CMB + C^sec. 
    ! C^sec terms  are CIB-C + CIB-P + radio + tSZ + kSZ + tSZ-CIB
    !----------------------------------------------------------------  

    do il=2,tt_lmax_k
       cltt_temp(il) = cltt(il) +&  !150 GHz only
            amp_c*cl_c(il)*(f2_dust/feff)**(2.d0*beta_c)*(planckratiod2*fluxtempd2)**2.d0&
            +amp_d*cl_p(il)*(f2_dust/feff)**(2.d0*beta_d)*(planckratiod2*fluxtempd2)**2.d0&
            +(amp_s2+9.2)*cl_p(il)*(f2_synch/feff)**(2.d0*alpha_s)*fluxtemps2**2.d0&
            +f2*f2/(f0*f0)*amp_tsz*cl_tsz(il)&
            +amp_ksz*cl_ksz(il)&
            -2.d0*sqrt(amp_c*amp_tsz)*xi*cl_szcib(il)*((f2_dust**beta_c*f2*planckratiod2*fluxtempd2)/(feff**beta_c*f0))
       
       ! Divide theory by calibration
       cltt_temp(il) =cltt_temp(il)/(cal_2**2.d0)
    enddo
    !Multiply by window functions
    btt_th(0:bmax0_k-1)=MATMUL(win_func(0:bmax0_k-1,2:tt_lmax_k),cltt_temp(2:tt_lmax_k))
    

    !--------------------------------------------------------------
    ! chi2 calculation
    !--------------------------------------------------------------

    like_sptk = 0.d0

    do i = 1, bmax0_k
       diffs(i,1) = btt_dat(i-1) - btt_th(i-1)
       diffs2(1,i) = diffs(i,1)
    enddo

    tmp(:,:) = matmul(inverse(:,:),diffs(:,:))
    chi2(:,:) = matmul(diffs2(:,:),tmp(:,:))

    like_sptk = like_sptk+chi2(1,1)/2.d0

   10  continue
    
  end SUBROUTINE spt_story_likelihood_compute  
  !===============================================================================
  
END MODULE spt_story_likelihood

