!================================================================
program test_likelihood

!================================================================
!ACT 3-year multi-frequency likelihood code
!E. Calabrese, J. Dunkley Jan 2013. 
!This version updated Oct 2013.
!Based on WMAP likelihood codes and ACT 1-year code
!================================================================

use highell_options
use highell_likelihood
use Highell_subroutines

implicit none

real(8), dimension(:), allocatable :: cl_tt
character(LEN=128) :: filename
real(8)            :: like_tot,like 
real(8)            :: amp_tsz,amp_ksz,xi,amp_d,amp_s,amp_s2,amp_c
real(8)            :: n,beta_c,beta_d,alpha_s,amp_gs,amp_ge
real(8)            :: cas1,cas2,cae1,cae2,cal_1,cal_2,cal_3
INTEGER, PARAMETER :: ttmax =6000 
integer            :: il,dummy,lun
!---------------------------------------------------

print *,""
print *,"High ell likelihood test"
print *,"==================================="

!---------------------------------------------------
! read in test Cls
!---------------------------------------------------
filename = 'data/wmap7_act_lcdm_bestfit_lensedCls_6000.dat'

write(*,*)"Reading in Cls from: ",trim(filename)
open(unit=lun,file=filename,action='read',status='old')

allocate(cl_tt(2:tt_lmax))
cl_tt(2:tt_lmax)=0.d0

!CMB theory Cls used out to l=6000 
do il=2,ttmax
   read(lun,*)dummy,cl_tt(il)
enddo
close(lun)

call highell_likelihood_init

!Set secondary parameters. All power parameteres are defined at l=3000, 150 GHz

amp_tsz =4.66d0  !tSZ power
amp_ksz =1.60d0  !kSZ power
xi      =0.09d0  !tSZ-CIB cross-correlation coefficient
amp_d   =6.87d0  !CIB Poisson power 
amp_s   =3.50d0  !Radio Poisson power in ACT
amp_s2  =1.37d0  !Radio Poisson power in SPT
amp_c   =6.10d0  !CIB clustered power
n       =1.20d0  !CIB spectral index in ell
beta_c  =2.08d0  !CIB clustered emissivity index (for T=9.7K)
beta_d  =2.08d0  !CIB Poisson emissivity index (for T=9.7K)
alpha_s =-0.5d0  !Radio spectral index in flux
amp_gs  =0.31d0  !ACT-S Cirrus power
amp_ge  =0.88d0  !ACT-E Cirrus power

cas1    =1.01d0  !ACT-S 148 GHz calibration
cas2    =1.02d0  !ACT-S 218 GHz calibration
cae1    =1.00d0  !ACT-E 148 GHz calibration
cae2    =0.99d0  !ACT-E 218 GHz calibration
cal_1   =1.00d0  !SPT 95 GHz calibration
cal_2   =1.01d0  !SPT 150 GHz calibration
cal_3   =1.02d0  !SPT 220 GHz calibration


call highell_likelihood_compute(cl_tt,amp_tsz,amp_ksz,xi,amp_d,amp_s,amp_s2,&
amp_c,n,beta_c,beta_d,alpha_s,amp_gs,amp_ge,cas1,cas2,cae1,cae2,cal_1,cal_2,&
cal_3,like_tot)

write(*,*) ''
print *,   "---------------------------------------------------------------"
print *,   'Total High ell -2lnL      =',2.d0*like_tot
print *,   "---------------------------------------------------------------"
write(*,*) ''
write(*,*) 'Expected -2lnL (ACT+SPT)  =   783.480997225846 for 827 data points'
write(*,*) '----------------------------------------------------------------'
write(*,*) 'Expected -2lnL (ACT-E)    =   240.385376700682 for 230 data points'
write(*,*) 'Expected -2lnL (ACT-S)    =   432.002728383558 for 480 data points'
write(*,*) ''
write(*,*) 'Expected -2lnL (SPT-high) =   72.2377661542505 for 90 data points'
write(*,*) 'Expected -2lnL (SPT-low)  =   38.8551259873550 for 27 data points'
write(*,*) '----------------------------------------------------------------'


end program test_likelihood
